@@ -27,7 +27,7 @@ package hudson;
 
 import hudson.Launcher.LocalLauncher;
 import hudson.Launcher.RemoteLauncher;
-import hudson.model.Hudson;
+import jenkins.model.Jenkins;
 import hudson.model.TaskListener;
 import hudson.model.AbstractProject;
 import hudson.model.Item;
@@ -39,6 +39,7 @@ import hudson.remoting.Pipe;
 import hudson.remoting.RemoteOutputStream;
 import hudson.remoting.VirtualChannel;
 import hudson.remoting.RemoteInputStream;
+import hudson.remoting.Which;
 import hudson.security.AccessControlled;
 import hudson.util.DirScanner;
 import hudson.util.IOException2;
@@ -85,6 +86,7 @@ import java.util.List;
 import java.util.StringTokenizer;
 import java.util.Arrays;
 import java.util.Comparator;
+import java.util.logging.Level;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 import java.util.concurrent.ExecutionException;
@@ -628,7 +630,7 @@ public final class FilePath implements Serializable {
                 listener.getLogger().println(message);
 
             // for HTTP downloads, enable automatic retry for added resilience
-            InputStream in = archive.getProtocol().equals("http") ? new RetryableHttpStream(archive) : con.getInputStream();
+            InputStream in = archive.getProtocol().startsWith("http") ? ProxyConfiguration.getInputStream(archive) : con.getInputStream();
             CountingInputStream cis = new CountingInputStream(in);
             try {
                 if(archive.toExternalForm().endsWith(".zip"))
@@ -753,7 +755,7 @@ public final class FilePath implements Serializable {
             }
         } else {
             // the file is on the local machine.
-            return callable.invoke(new File(remote), Hudson.MasterComputer.localChannel);
+            return callable.invoke(new File(remote), Jenkins.MasterComputer.localChannel);
         }
     }
 
@@ -763,7 +765,7 @@ public final class FilePath implements Serializable {
      */
     public <T> Future<T> actAsync(final FileCallable<T> callable) throws IOException, InterruptedException {
         try {
-            return (channel!=null ? channel : Hudson.MasterComputer.localChannel)
+            return (channel!=null ? channel : Jenkins.MasterComputer.localChannel)
                 .callAsync(new FileCallableWrapper<T>(callable));
         } catch (IOException e) {
             // wrap it into a new IOException so that we get the caller's stack trace as well.
@@ -1209,6 +1211,7 @@ public final class FilePath implements Serializable {
      *      See {@link FileSet} for the syntax. String like "foo/*.zip" or "foo/*&#42;/*.xml"
      * @return
      *      can be empty but always non-null.
+     * @since 1.407
      */
     public FilePath[] list(final String includes, final String excludes) throws IOException, InterruptedException {
         return act(new FileCallable<FilePath[]>() {
@@ -1419,8 +1422,29 @@ public final class FilePath implements Serializable {
         });
 
         // make sure the write fully happens before we return.
-        if (channel!=null)
-            channel.syncLocalIO();
+        syncIO();
+    }
+
+    private void syncIO() throws InterruptedException {
+        try {
+            if (channel!=null)
+                _syncIO();
+        } catch (AbstractMethodError e) {
+            // legacy slave.jar. Handle this gracefully
+            try {
+                LOGGER.log(Level.WARNING,"Looks like an old slave.jar. Please update "+ Which.jarFile(Channel.class)+" to the new version",e);
+            } catch (IOException _) {
+                // really ignore this time
+            }
+        }
+    }
+
+    /**
+     * A pointless function to work around what appears to be a HotSpot problem. See JENKINS-5756 and bug 6933067
+     * on BugParade for more details.
+     */
+    private void _syncIO() throws InterruptedException {
+        channel.syncLocalIO();
     }
 
     /**
@@ -1440,12 +1464,26 @@ public final class FilePath implements Serializable {
 
     /**
      * Copies the contents of this directory recursively into the specified target directory.
+     * 
+     * @return
+     *      the number of files copied.
      * @since 1.312 
      */
     public int copyRecursiveTo(FilePath target) throws IOException, InterruptedException {
         return copyRecursiveTo("**/*",target);
     }
 
+    /**
+     * Copies the files that match the given file mask to the specified target node.
+     *
+     * @param fileMask
+     *      Ant GLOB pattern.
+     *      String like "foo/bar/*.xml" Multiple patterns can be separated
+     *      by ',', and whitespace can surround ',' (so that you can write
+     *      "abc, def" and "abc,def" to mean the same thing.
+     * @return
+     *      the number of files copied.
+     */
     public int copyRecursiveTo(String fileMask, FilePath target) throws IOException, InterruptedException {
         return copyRecursiveTo(fileMask,null,target);
     }
@@ -1872,7 +1910,7 @@ public final class FilePath implements Serializable {
     private static void checkPermissionForValidate() {
         AccessControlled subject = Stapler.getCurrentRequest().findAncestorObject(AbstractProject.class);
         if (subject == null)
-            Hudson.getInstance().checkPermission(Hudson.ADMINISTER);
+            Jenkins.getInstance().checkPermission(Jenkins.ADMINISTER);
         else
             subject.checkPermission(Item.CONFIGURE);
     }
@@ -1896,7 +1934,7 @@ public final class FilePath implements Serializable {
 
     public VirtualChannel getChannel() {
         if(channel!=null)   return channel;
-        else                return Hudson.MasterComputer.localChannel;
+        else                return Jenkins.MasterComputer.localChannel;
     }
 
     /**
