@@ -25,6 +25,7 @@
 package hudson.model;
 
 import com.infradna.tool.bridge_method_injector.WithBridgeMethods;
+import hudson.AbortException;
 import hudson.XmlFile;
 import hudson.Util;
 import hudson.Functions;
@@ -40,6 +41,8 @@ import hudson.util.AlternativeUiTextProvider;
 import hudson.util.AlternativeUiTextProvider.Message;
 import hudson.util.AtomicFileWriter;
 import hudson.util.IOException2;
+import hudson.util.IOUtils;
+import jenkins.model.Jenkins;
 import org.apache.tools.ant.taskdefs.Copy;
 import org.apache.tools.ant.types.FileSet;
 import org.kohsuke.stapler.WebMethod;
@@ -48,7 +51,11 @@ import org.kohsuke.stapler.export.ExportedBean;
 
 import java.io.File;
 import java.io.IOException;
+import java.util.ArrayList;
 import java.util.Collection;
+import java.util.HashMap;
+import java.util.List;
+import java.util.Map;
 
 import org.kohsuke.stapler.StaplerRequest;
 import org.kohsuke.stapler.StaplerResponse;
@@ -121,7 +128,7 @@ public abstract class AbstractItem extends Actionable implements Item, HttpDelet
     /**
      * This bridge method is to maintain binary compatibility with {@link TopLevelItem#getParent()}.
      */
-    @WithBridgeMethods(value=Hudson.class,castRequired=true)
+    @WithBridgeMethods(value=Jenkins.class,castRequired=true)
     public ItemGroup getParent() {
         assert parent!=null;
         return parent;
@@ -283,6 +290,44 @@ public abstract class AbstractItem extends Actionable implements Item, HttpDelet
         else                return n+" \u00BB "+getDisplayName();
     }
 
+    public String getRelativeNameFrom(ItemGroup p) {
+        // first list up all the parents
+        Map<ItemGroup,Integer> parents = new HashMap<ItemGroup,Integer>();
+        int depth=0;
+        while (p!=null) {
+            parents.put(p, depth++);
+            if (p instanceof Item)
+                p = ((Item)p).getParent();
+            else
+                p = null;
+        }
+
+        StringBuilder buf = new StringBuilder();
+        Item i=this;
+        while (true) {
+            if (buf.length()>0) buf.insert(0,'/');
+            buf.insert(0,i.getName());
+            ItemGroup g = i.getParent();
+
+            Integer d = parents.get(g);
+            if (d!=null) {
+                String s="";
+                for (int j=d; j>0; j--)
+                    s+="../";
+                return s+buf;
+            }
+
+            if (g instanceof Item)
+                i = (Item)g;
+            else
+                return null;
+        }
+    }
+
+    public String getRelativeNameFrom(Item item) {
+        return getRelativeNameFrom(item.getParent());
+    }
+
     /**
      * Called right after when a {@link Item} is loaded from disk.
      * This is an opporunity to do a post load processing.
@@ -297,6 +342,12 @@ public abstract class AbstractItem extends Actionable implements Item, HttpDelet
      * the files are first copied on the file system,
      * then it will be loaded, then this method will be invoked
      * to perform any implementation-specific work.
+     *
+     * <p>
+     * 
+     *
+     * @param src
+     *      Item from which it's copied from. The same type as {@code this}. Never null.
      */
     public void onCopiedFrom(Item src) {
     }
@@ -326,10 +377,10 @@ public abstract class AbstractItem extends Actionable implements Item, HttpDelet
 
     @Exported(visibility=999,name="url")
     public final String getAbsoluteUrl() {
-        StaplerRequest request = Stapler.getCurrentRequest();
-        if(request==null)
-            throw new IllegalStateException("Not processing a HTTP request");
-        return Util.encode(Hudson.getInstance().getRootUrl()+getUrl());
+        String r = Jenkins.getInstance().getRootUrl();
+        if(r==null)
+            throw new IllegalStateException("Root URL isn't configured yet. Cannot compute absolute URL.");
+        return Util.encode(r+getUrl());
     }
 
     /**
@@ -343,7 +394,7 @@ public abstract class AbstractItem extends Actionable implements Item, HttpDelet
      * Returns the {@link ACL} for this object.
      */
     public ACL getACL() {
-        return Hudson.getInstance().getAuthorizationStrategy().getACL(this);
+        return Jenkins.getInstance().getAuthorizationStrategy().getACL(this);
     }
 
     /**
@@ -374,7 +425,7 @@ public abstract class AbstractItem extends Actionable implements Item, HttpDelet
     }
 
     public Descriptor getDescriptorByName(String className) {
-        return Hudson.getInstance().getDescriptorByName(className);
+        return Jenkins.getInstance().getDescriptorByName(className);
     }
 
     /**
@@ -409,6 +460,10 @@ public abstract class AbstractItem extends Actionable implements Item, HttpDelet
 
     /**
      * Deletes this item.
+     *
+     * <p>
+     * Any exception indicates the deletion has failed, but {@link AbortException} would prevent the caller
+     * from showing the stack trace. This
      */
     public synchronized void delete() throws IOException, InterruptedException {
         checkPermission(DELETE);
@@ -420,7 +475,7 @@ public abstract class AbstractItem extends Actionable implements Item, HttpDelet
             // ignore
         }
 
-        Hudson.getInstance().rebuildDependencyGraph();
+        Jenkins.getInstance().rebuildDependencyGraph();
     }
 
     /**
@@ -448,8 +503,8 @@ public abstract class AbstractItem extends Actionable implements Item, HttpDelet
         if (req.getMethod().equals("GET")) {
             // read
             checkPermission(EXTENDED_READ);
-            rsp.setContentType("application/xml;charset=UTF-8");
-            getConfigFile().writeRawTo(rsp.getWriter());
+            rsp.setContentType("application/xml");
+            IOUtils.copy(getConfigFile().getFile(),rsp.getOutputStream());
             return;
         }
         if (req.getMethod().equals("POST")) {
@@ -497,7 +552,7 @@ public abstract class AbstractItem extends Actionable implements Item, HttpDelet
     @CLIResolver
     public static AbstractItem resolveForCLI(
             @Argument(required=true,metaVar="NAME",usage="Job name") String name) throws CmdLineException {
-        AbstractItem item = Hudson.getInstance().getItemByFullName(name, AbstractItem.class);
+        AbstractItem item = Jenkins.getInstance().getItemByFullName(name, AbstractItem.class);
         if (item==null)
             throw new CmdLineException(null,Messages.AbstractItem_NoSuchJobExists(name,AbstractProject.findNearest(name).getFullName()));
         return item;
