@@ -1,19 +1,19 @@
 /*
  * The MIT License
- * 
+ *
  * Copyright (c) 2004-2010, Sun Microsystems, Inc., Kohsuke Kawaguchi,
  * Martin Eigenbrodt. Seiji Sogabe, Alan Harder
- * 
+ *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
  * in the Software without restriction, including without limitation the rights
  * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  * copies of the Software, and to permit persons to whom the Software is
  * furnished to do so, subject to the following conditions:
- * 
+ *
  * The above copyright notice and this permission notice shall be included in
  * all copies or substantial portions of the Software.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
@@ -24,8 +24,10 @@
  */
 package hudson.model;
 
+import com.google.common.collect.ImmutableList;
 import hudson.security.ACL;
 import hudson.security.NotSerilizableSecurityContext;
+import jenkins.model.Jenkins;
 import org.acegisecurity.context.SecurityContext;
 import org.acegisecurity.context.SecurityContextHolder;
 
@@ -61,14 +63,14 @@ import java.util.Stack;
  * there's a change (which is relatively rare), a new instance
  * will be created. This eliminates the need of synchronization.
  *
- * @see Hudson#getDependencyGraph() 
+ * @see Jenkins#getDependencyGraph()
  * @author Kohsuke Kawaguchi
  */
-public final class DependencyGraph implements Comparator<AbstractProject> {
+public class DependencyGraph implements Comparator<AbstractProject> {
 
     private Map<AbstractProject, List<DependencyGroup>> forward = new HashMap<AbstractProject, List<DependencyGroup>>();
     private Map<AbstractProject, List<DependencyGroup>> backward = new HashMap<AbstractProject, List<DependencyGroup>>();
-    
+
     private transient Map<Class<?>, Object> computationalData;
 
     private boolean built;
@@ -77,6 +79,9 @@ public final class DependencyGraph implements Comparator<AbstractProject> {
      * Builds the dependency graph.
      */
     public DependencyGraph() {
+    }
+    
+    public void build() {
         // Set full privileges while computing to avoid missing any projects the current user cannot see.
         // Use setContext (NOT getContext().setAuthentication()) so we don't affect concurrent threads for same HttpSession.
         SecurityContext saveCtx = SecurityContextHolder.getContext();
@@ -85,7 +90,7 @@ public final class DependencyGraph implements Comparator<AbstractProject> {
             NotSerilizableSecurityContext system = new NotSerilizableSecurityContext();
             system.setAuthentication(ACL.SYSTEM);
             SecurityContextHolder.setContext(system);
-            for( AbstractProject p : Hudson.getInstance().getAllItems(AbstractProject.class) )
+            for( AbstractProject p : getAllProjects() )
                 p.buildDependencyGraph(this);
 
             forward = finalize(forward);
@@ -97,6 +102,10 @@ public final class DependencyGraph implements Comparator<AbstractProject> {
             SecurityContextHolder.setContext(saveCtx);
         }
     }
+    
+    Collection<AbstractProject> getAllProjects() {
+        return Jenkins.getInstance().getAllItems(AbstractProject.class);
+    }
 
     /**
      * Special constructor for creating an empty graph
@@ -113,7 +122,7 @@ public final class DependencyGraph implements Comparator<AbstractProject> {
     public <T> void putComputationalData(Class<T> key, T value) {
         this.computationalData.put(key, value);
     }
-    
+
     /**
      * Gets temporary data which is needed for building up the dependency graph.
      */
@@ -122,7 +131,7 @@ public final class DependencyGraph implements Comparator<AbstractProject> {
         T result = (T) this.computationalData.get(key);
         return result;
     }
-    
+
     /**
      * Gets all the immediate downstream projects (IOW forward edges) of the given project.
      *
@@ -147,7 +156,7 @@ public final class DependencyGraph implements Comparator<AbstractProject> {
         List<DependencyGroup> v = map.get(src);
         if(v==null) return Collections.emptyList();
         List<AbstractProject> result = new ArrayList<AbstractProject>(v.size());
-        for (Dependency d : v) result.add(up ? d.getUpstreamProject() : d.getDownstreamProject());
+        for (DependencyGroup d : v) result.add(up ? d.getUpstreamProject() : d.getDownstreamProject());
         return result;
     }
 
@@ -167,8 +176,16 @@ public final class DependencyGraph implements Comparator<AbstractProject> {
 
     private List<Dependency> get(Map<AbstractProject, List<DependencyGroup>> map, AbstractProject src) {
         List<DependencyGroup> v = map.get(src);
-        if(v!=null) return Collections.<Dependency>unmodifiableList(v);
-        else        return Collections.emptyList();
+        if(v==null) {
+            return ImmutableList.of();
+        } else {
+            ImmutableList.Builder<Dependency> builder = ImmutableList.builder();
+            for (DependencyGroup dependencyGroup : v) {
+                builder.addAll(dependencyGroup.getGroup());
+            }
+            return builder.build();
+        }
+
     }
 
     /**
@@ -301,101 +318,8 @@ public final class DependencyGraph implements Comparator<AbstractProject> {
         return Collections.unmodifiableMap(m);
     }
 
-//    /**
-//     * Experimental visualization of project dependencies.
-//     */
-//    public void doGraph( StaplerRequest req, StaplerResponse rsp ) throws IOException {
-//        // Require admin permission for now (avoid exposing project names with restricted permissions)
-//        Hudson.getInstance().checkPermission(Hudson.ADMINISTER);
-//        try {
-//
-//            // creates a dummy graphics just so that we can measure font metrics
-//            BufferedImage emptyImage = new BufferedImage(1,1, BufferedImage.TYPE_INT_RGB );
-//            Graphics2D graphics = emptyImage.createGraphics();
-//            graphics.setFont(FONT);
-//            final FontMetrics fontMetrics = graphics.getFontMetrics();
-//
-//            // TODO: timestamp check
-//            Layout<AbstractProject> layout = new Layout<AbstractProject>(new Navigator<AbstractProject>() {
-//                public Collection<AbstractProject> vertices() {
-//                    // only include projects that have some dependency
-//                    List<AbstractProject> r = new ArrayList<AbstractProject>();
-//                    for (AbstractProject p : Hudson.getInstance().getAllItems(AbstractProject.class)) {
-//                        if(!getDownstream(p).isEmpty() || !getUpstream(p).isEmpty())
-//                            r.add(p);
-//                    }
-//                    return r;
-//                }
-//
-//                public Collection<AbstractProject> edge(AbstractProject p) {
-//                    return getDownstream(p);
-//                }
-//
-//                public Dimension getSize(AbstractProject p) {
-//                    int w = fontMetrics.stringWidth(p.getDisplayName()) + MARGIN*2;
-//                    return new Dimension(w, fontMetrics.getHeight() + MARGIN*2);
-//                }
-//            }, Direction.LEFTRIGHT);
-//
-//            Rectangle area = layout.calcDrawingArea();
-//            area.grow(4,4); // give it a bit of margin
-//            BufferedImage image = new BufferedImage(area.width, area.height, BufferedImage.TYPE_INT_RGB );
-//            Graphics2D g2 = image.createGraphics();
-//            g2.setTransform(AffineTransform.getTranslateInstance(-area.x,-area.y));
-//            g2.setPaint(Color.WHITE);
-//            g2.fill(area);
-//            g2.setFont(FONT);
-//
-//            g2.setPaint(Color.BLACK);
-//            for( AbstractProject p : layout.vertices() ) {
-//                final Point sp = center(layout.vertex(p));
-//
-//                for (AbstractProject q : layout.edges(p)) {
-//                    Point cur=sp;
-//                    for( Point pt : layout.edge(p,q) ) {
-//                        g2.drawLine(cur.x, cur.y, pt.x, pt.y);
-//                        cur=pt;
-//                    }
-//
-//                    final Point ep = center(layout.vertex(q));
-//                    g2.drawLine(cur.x, cur.y, ep.x, ep.y);
-//                }
-//            }
-//
-//            int diff = fontMetrics.getAscent()+fontMetrics.getLeading()/2;
-//
-//            for( AbstractProject p : layout.vertices() ) {
-//                Rectangle r = layout.vertex(p);
-//                g2.setPaint(Color.WHITE);
-//                g2.fillRect(r.x, r.y, r.width, r.height);
-//                g2.setPaint(Color.BLACK);
-//                g2.drawRect(r.x, r.y, r.width, r.height);
-//                g2.drawString(p.getDisplayName(), r.x+MARGIN, r.y+MARGIN+ diff);
-//            }
-//
-//            rsp.setContentType("image/png");
-//            ServletOutputStream os = rsp.getOutputStream();
-//            ImageIO.write(image, "PNG", os);
-//            os.close();
-//        } catch(HeadlessException e) {
-//            // not available. send out error message
-//            rsp.sendRedirect2(req.getContextPath()+"/images/headless.png");
-//        }
-//    }
-//
-//    private Point center(Rectangle r) {
-//        return new Point(r.x+r.width/2,r.y+r.height/2);
-//    }
-//
-//    private static final Font FONT = new Font("TimesRoman", Font.PLAIN, 10);
-//    /**
-//     * Margins between the project name and its bounding box.
-//     */
-//    private static final int MARGIN = 4;
-
-
-    private static final Comparator<Dependency> NAME_COMPARATOR = new Comparator<Dependency>() {
-        public int compare(Dependency lhs, Dependency rhs) {
+    private static final Comparator<DependencyGroup> NAME_COMPARATOR = new Comparator<DependencyGroup>() {
+        public int compare(DependencyGroup lhs, DependencyGroup rhs) {
             int cmp = lhs.getUpstreamProject().getName().compareTo(rhs.getUpstreamProject().getName());
             return cmp != 0 ? cmp : lhs.getDownstreamProject().getName().compareTo(rhs.getDownstreamProject().getName());
         }
@@ -410,10 +334,10 @@ public final class DependencyGraph implements Comparator<AbstractProject> {
         Set<AbstractProject> o1sdownstreams = getTransitiveDownstream(o1);
         Set<AbstractProject> o2sdownstreams = getTransitiveDownstream(o2);
         if (o1sdownstreams.contains(o2)) {
-            if (o2sdownstreams.contains(o1)) return 0; else return 1;                       
+            if (o2sdownstreams.contains(o1)) return 0; else return 1;
         } else {
-            if (o2sdownstreams.contains(o1)) return -1; else return 0; 
-        }               
+            if (o2sdownstreams.contains(o1)) return -1; else return 0;
+        }
     }
 
     /**
@@ -478,30 +402,31 @@ public final class DependencyGraph implements Comparator<AbstractProject> {
     /**
      * Collect multiple dependencies between the same two projects.
      */
-    private static class DependencyGroup extends Dependency {
+    private static class DependencyGroup {
         private Set<Dependency> group = new LinkedHashSet<Dependency>();
 
         DependencyGroup(Dependency first) {
-            super(first.getUpstreamProject(), first.getDownstreamProject());
+            this.upstream = first.getUpstreamProject();
+            this.downstream= first.getDownstreamProject();
             group.add(first);
         }
 
-        void add(Dependency next) {
+        private void add(Dependency next) {
             group.add(next);
         }
 
-        @Override
-        public boolean shouldTriggerBuild(AbstractBuild build, TaskListener listener,
-                                          List<Action> actions) {
-            List<Action> check = new ArrayList<Action>();
-            for (Dependency d : group) {
-                if (d.shouldTriggerBuild(build, listener, check)) {
-                    actions.addAll(check);
-                    return true;
-                } else
-                    check.clear();
-            }
-            return false;
+        public Set<Dependency> getGroup() {
+            return group;
+        }
+
+        private AbstractProject upstream, downstream;
+
+        public AbstractProject getUpstreamProject() {
+            return upstream;
+        }
+
+        public AbstractProject getDownstreamProject() {
+            return downstream;
         }
     }
 }
