@@ -2,6 +2,7 @@ package hudson.model;
 
 import hudson.ExtensionList;
 import hudson.ExtensionPoint;
+import jenkins.model.Jenkins;
 
 import java.util.ArrayList;
 import java.util.List;
@@ -25,7 +26,7 @@ public abstract class TransientViewActionFactory implements ExtensionPoint {
      * Returns all the registered {@link TransientViewActionFactory}s.
      */
 	public static ExtensionList<TransientViewActionFactory> all() {
-		return Hudson.getInstance().getExtensionList(TransientViewActionFactory.class);
+		return Jenkins.getInstance().getExtensionList(TransientViewActionFactory.class);
 	}
 	
     /**
