@@ -34,6 +34,7 @@ import hudson.model.Queue.BuildableItem;
 import hudson.model.Queue.Executable;
 import hudson.model.Queue.JobOffer;
 import hudson.model.Queue.Task;
+import hudson.model.labels.LabelAssignmentAction;
 
 import java.util.AbstractList;
 import java.util.ArrayList;
@@ -181,7 +182,7 @@ public class MappingWorksheet {
             super(base);
             assert !base.isEmpty();
             this.index = index;
-            this.assignedLabel = base.get(0).getAssignedLabel();
+            this.assignedLabel = getAssignedLabel(base.get(0));
 
             Node lbo = base.get(0).getLastBuiltOn();
             for (ExecutorChunk ec : executors) {
@@ -193,6 +194,14 @@ public class MappingWorksheet {
             lastBuiltOn = null;
         }
 
+        private Label getAssignedLabel(SubTask task) {
+            for (LabelAssignmentAction laa : item.getActions(LabelAssignmentAction.class)) {
+                Label l = laa.getAssignedLabel(task);
+                if (l!=null)    return l;
+            }
+            return task.getAssignedLabel();
+        }
+
         public List<ExecutorChunk> applicableExecutorChunks() {
             List<ExecutorChunk> r = new ArrayList<ExecutorChunk>(executors.size());
             for (ExecutorChunk e : executors) {
