@@ -37,7 +37,7 @@ import hudson.model.BuildListener;
 import hudson.model.Computer;
 import hudson.model.EnvironmentSpecific;
 import hudson.model.Node;
-import hudson.model.Hudson;
+import jenkins.model.Jenkins;
 import hudson.model.TaskListener;
 import hudson.remoting.Callable;
 import hudson.remoting.VirtualChannel;
@@ -309,7 +309,7 @@ public class Maven extends Builder {
 
     /**
      * @deprecated as of 1.286
-     *      Use {@link Hudson#getDescriptorByType(Class)} to obtain the current instance.
+     *      Use {@link jenkins.model.Jenkins#getDescriptorByType(Class)} to obtain the current instance.
      *      For compatibility, this field retains the last created {@link DescriptorImpl}.
      *      TODO: fix sonar plugin that depends on this. That's the only plugin that depends on this field.
      */
@@ -381,7 +381,7 @@ public class Maven extends Builder {
 
         @DataBoundConstructor
         public MavenInstallation(String name, String home, List<? extends ToolProperty<?>> properties) {
-            super(name, home, properties);
+            super(Util.fixEmptyAndTrim(name), Util.fixEmptyAndTrim(home), properties);
         }
 
         /**
@@ -524,12 +524,12 @@ public class Maven extends Builder {
 
             @Override
             public MavenInstallation[] getInstallations() {
-                return Hudson.getInstance().getDescriptorByType(Maven.DescriptorImpl.class).getInstallations();
+                return Jenkins.getInstance().getDescriptorByType(Maven.DescriptorImpl.class).getInstallations();
             }
 
             @Override
             public void setInstallations(MavenInstallation... installations) {
-                Hudson.getInstance().getDescriptorByType(Maven.DescriptorImpl.class).setInstallations(installations);
+                Jenkins.getInstance().getDescriptorByType(Maven.DescriptorImpl.class).setInstallations(installations);
             }
 
             /**
@@ -537,7 +537,7 @@ public class Maven extends Builder {
              */
             public FormValidation doCheckMavenHome(@QueryParameter File value) {
                 // this can be used to check the existence of a file on the server, so needs to be protected
-                if(!Hudson.getInstance().hasPermission(Hudson.ADMINISTER))
+                if(!Jenkins.getInstance().hasPermission(Jenkins.ADMINISTER))
                     return FormValidation.ok();
 
                 if(value.getPath().equals(""))
