@@ -31,7 +31,7 @@ import hudson.model.AbstractBuild;
 import hudson.model.AbstractProject;
 import hudson.model.Action;
 import hudson.model.Cause;
-import hudson.model.Hudson;
+import jenkins.model.Jenkins;
 import hudson.model.Item;
 import hudson.model.Project;
 import hudson.model.SCMedItem;
@@ -91,7 +91,7 @@ public class SCMTrigger extends Trigger<SCMedItem> {
      * @since 1.375
      */
     public void run(Action[] additionalActions) {
-        if(Hudson.getInstance().isQuietingDown())
+        if(Jenkins.getInstance().isQuietingDown())
             return; // noop
 
         DescriptorImpl d = getDescriptor();
@@ -285,7 +285,7 @@ public class SCMTrigger extends Trigger<SCMedItem> {
         }
 
         public String getIconFileName() {
-            return "clipboard.gif";
+            return "clipboard.png";
         }
 
         public String getDisplayName() {
@@ -329,7 +329,7 @@ public class SCMTrigger extends Trigger<SCMedItem> {
         }
 
         public String getIconFileName() {
-            return "clipboard.gif";
+            return "clipboard.png";
         }
 
         public String getDisplayName() {
