@@ -28,7 +28,7 @@ import com.sun.jna.Native;
 import com.sun.jna.ptr.IntByReference;
 import hudson.EnvVars;
 import hudson.Util;
-import hudson.model.Hudson;
+import jenkins.model.Jenkins;
 import hudson.remoting.Callable;
 import hudson.remoting.Channel;
 import hudson.remoting.VirtualChannel;
@@ -277,7 +277,7 @@ public abstract class ProcessTree implements Iterable<OSProcess>, IProcessTree,
          * Executes a chunk of code at the same machine where this process resides.
          */
         public <T> T act(ProcessCallable<T> callable) throws IOException, InterruptedException {
-            return callable.invoke(this,Hudson.MasterComputer.localChannel);
+            return callable.invoke(this, Jenkins.MasterComputer.localChannel);
         }
 
         Object writeReplace() {
@@ -1053,10 +1053,14 @@ public abstract class ProcessTree implements Iterable<OSProcess>, IProcessTree,
                         */
 
                     int nargs = m.readInt();
-                    m.readString(); // exec path
-                    for( int i=0; i<nargs; i++) {
-                        m.skip0();
-                        arguments.add(m.readString());
+                    String args0 = m.readString(); // exec path
+                    try {
+                        for( int i=0; i<nargs; i++) {
+                            m.skip0();
+                            arguments.add(m.readString());
+                        }
+                    } catch (IndexOutOfBoundsException e) {
+                        throw new IllegalStateException("Failed to parse arguments: arg0="+args0+", arguments="+arguments+", nargs="+nargs,e);
                     }
 
                     // this is how you can read environment variables
