@@ -27,12 +27,12 @@ import hudson.Launcher;
 import hudson.maven.MavenBuild.ProxyImpl2;
 import hudson.maven.util.ExecutionEventLogger;
 import hudson.model.BuildListener;
-import hudson.model.Hudson;
+import hudson.model.Executor;
+import jenkins.model.Jenkins;
 import hudson.model.Result;
 import hudson.remoting.Channel;
 import hudson.remoting.DelegatingCallable;
 import hudson.remoting.Future;
-import hudson.util.CopyOnWriteList;
 import hudson.util.IOException2;
 
 import java.io.IOException;
@@ -40,7 +40,6 @@ import java.io.PrintStream;
 import java.io.Serializable;
 import java.lang.reflect.InvocationTargetException;
 import java.text.NumberFormat;
-import java.util.ArrayList;
 import java.util.Date;
 import java.util.HashMap;
 import java.util.HashSet;
@@ -51,6 +50,7 @@ import java.util.Map.Entry;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.CopyOnWriteArrayList;
 import java.util.concurrent.ExecutionException;
+import java.util.logging.Logger;
 
 import org.apache.maven.cli.PrintStreamLogger;
 import org.apache.maven.execution.AbstractExecutionListener;
@@ -114,12 +114,7 @@ public class Maven3Builder extends AbstractMavenBuilder implements DelegatingCal
             
             markAsSuccess = false;
 
-            // working around NPE when someone puts a null value into systemProps.
-            for (Map.Entry<String,String> e : systemProps.entrySet()) {
-                if (e.getValue()==null)
-                    throw new IllegalArgumentException("System property "+e.getKey()+" has a null value");
-                System.getProperties().put(e.getKey(), e.getValue());
-            }
+            registerSystemProperties();
 
             listener.getLogger().println(formatArgs(goals));
             
@@ -133,20 +128,17 @@ public class Maven3Builder extends AbstractMavenBuilder implements DelegatingCal
                 try {
                     if(!f.isDone() && !messageReported) {
                         messageReported = true;
-                        // FIXME messages
-                        listener.getLogger().println("maven builder waiting");
+                        listener.getLogger().println(Messages.MavenBuilder_Waiting());
                     }
                     f.get();
                 } catch (InterruptedException e) {
                     // attempt to cancel all asynchronous tasks
                     for (Future<?> g : futures)
                         g.cancel(true);
-                    // FIXME messages
-                    listener.getLogger().println("build aborted");
-                    return Result.ABORTED;
+                    listener.getLogger().println(Messages.MavenBuilder_Aborted());
+                    return Executor.currentExecutor().abortResult();
                 } catch (ExecutionException e) {
-                    // FIXME messages
-                    e.printStackTrace(listener.error("async build failed"));
+                    e.printStackTrace(listener.error(Messages.MavenBuilder_AsyncFailed()));
                 }
             }
             mavenExecutionListener.overheadTime += System.nanoTime()-startTime;
@@ -170,6 +162,11 @@ public class Maven3Builder extends AbstractMavenBuilder implements DelegatingCal
             if (!mavenExecutionResult.getThrowables().isEmpty()) {
                 logger.println( "mavenExecutionResult exceptions not empty");
                 for(Throwable throwable : mavenExecutionResult.getThrowables()) {
+                    logger.println("message : " + throwable.getMessage());
+                    if (throwable.getCause()!=null) {
+                        logger.println("cause : " + throwable.getCause().getMessage());
+                    }
+                    logger.println("Stack trace : ");
                     throwable.printStackTrace( logger );
                 }
                 
@@ -195,7 +192,7 @@ public class Maven3Builder extends AbstractMavenBuilder implements DelegatingCal
 
     // since reporters might be from plugins, use the uberjar to resolve them.
     public ClassLoader getClassLoader() {
-        return Hudson.getInstance().getPluginManager().uberClassLoader;
+        return Jenkins.getInstance().getPluginManager().uberClassLoader;
     }
     
     
@@ -282,13 +279,6 @@ public class Maven3Builder extends AbstractMavenBuilder implements DelegatingCal
             return null;
         }
         
-        // FIME really used somewhere ???
-        // FIXME MojoInfo need the real mojo ?? 
-        // so tricky to do. need to use MavenPluginManager on the current Maven Build
-        private Mojo getMojo(MojoExecution mojoExecution, MavenSession mavenSession) {
-            return null;
-        }
-        
         private ExpressionEvaluator getExpressionEvaluator(MavenSession session, MojoExecution mojoExecution) {
             return new PluginParameterExpressionEvaluator( session, mojoExecution );
         }
@@ -324,7 +314,7 @@ public class Maven3Builder extends AbstractMavenBuilder implements DelegatingCal
             // set all modules which are not actually being build (in incremental builds) to NOT_BUILD
             // JENKINS-9072
             List<MavenProject> projects = event.getSession().getProjects();
-            //maven3Builder.listener.getLogger().println("Projects to build: " + projects);
+            debug("Projects to build: " + projects);
             Set<ModuleName> buildingProjects = new HashSet<ModuleName>();
             for (MavenProject p : projects) {
                 buildingProjects.add(new ModuleName(p));
@@ -345,7 +335,7 @@ public class Maven3Builder extends AbstractMavenBuilder implements DelegatingCal
          * @see org.apache.maven.execution.ExecutionListener#sessionEnded(org.apache.maven.execution.ExecutionEvent)
          */
         public void sessionEnded( ExecutionEvent event )  {
-            maven3Builder.listener.getLogger().println( "sessionEnded" );
+            debug( "sessionEnded" );
             this.eventLogger.sessionEnded( event );
         }
 
@@ -353,7 +343,7 @@ public class Maven3Builder extends AbstractMavenBuilder implements DelegatingCal
          * @see org.apache.maven.execution.ExecutionListener#projectSkipped(org.apache.maven.execution.ExecutionEvent)
          */
         public void projectSkipped( ExecutionEvent event ) {
-            maven3Builder.listener.getLogger().println("projectSkipped " + event.getProject().getGroupId() 
+            debug("projectSkipped " + event.getProject().getGroupId()
                                                        + ":"  + event.getProject().getArtifactId()
                                                        + ":" + event.getProject().getVersion());    
             this.eventLogger.projectSkipped( event );
@@ -363,14 +353,14 @@ public class Maven3Builder extends AbstractMavenBuilder implements DelegatingCal
          * @see org.apache.maven.execution.ExecutionListener#projectStarted(org.apache.maven.execution.ExecutionEvent)
          */
         public void projectStarted( ExecutionEvent event ) {
-            //maven3Builder.listener.getLogger().println( "projectStarted " + event.getProject().getGroupId() + ":"
-            //                                                + event.getProject().getArtifactId() + ":" + event.getProject().getVersion() );
-            reccordProjectStarted( event );        
+            debug( "projectStarted " + event.getProject().getGroupId() + ":"
+                                                            + event.getProject().getArtifactId() + ":" + event.getProject().getVersion() );
+            recordProjectStarted(event);
             this.eventLogger.projectStarted( event );
             
         }
         
-        public void reccordProjectStarted( ExecutionEvent event ) {
+        private void recordProjectStarted(ExecutionEvent event) {
             MavenProject mavenProject = event.getProject();
             List<MavenReporter> mavenReporters = getMavenReporters( mavenProject );                
             
@@ -408,17 +398,16 @@ public class Maven3Builder extends AbstractMavenBuilder implements DelegatingCal
          * @see org.apache.maven.execution.ExecutionListener#projectSucceeded(org.apache.maven.execution.ExecutionEvent)
          */
         public void projectSucceeded( ExecutionEvent event ) {
-            maven3Builder.listener.getLogger().println( "projectSucceeded "
+            debug( "projectSucceeded "
                                                             + event.getProject().getGroupId() + ":"
                                                             + event.getProject().getArtifactId() + ":"
                                                             + event.getProject().getVersion());
-            reccordProjectSucceeded( event );
+            recordProjectSucceeded(event);
             this.eventLogger.projectSucceeded( event );
         }
         
-        public void reccordProjectSucceeded( ExecutionEvent event ) {
+        private void recordProjectSucceeded(ExecutionEvent event) {
             MavenBuildProxy2 mavenBuildProxy2 = getMavenBuildProxy2( event.getProject() );
-            mavenBuildProxy2.end();
             mavenBuildProxy2.setResult( Result.SUCCESS );
             
             
@@ -448,22 +437,22 @@ public class Maven3Builder extends AbstractMavenBuilder implements DelegatingCal
                 }
             }
            
-        }        
+            mavenBuildProxy2.end();
+        }
 
         /**
          * @see org.apache.maven.execution.ExecutionListener#projectFailed(org.apache.maven.execution.ExecutionEvent)
          */
         public void projectFailed( ExecutionEvent event ) {
-            maven3Builder.listener.getLogger().println("projectFailed " + event.getProject().getGroupId() 
+            debug("projectFailed " + event.getProject().getGroupId()
                                                                         + ":"  + event.getProject().getArtifactId()
                                                                         + ":" + event.getProject().getVersion());
-            reccordProjectFailed( event );
+            recordProjectFailed(event);
             this.eventLogger.projectFailed( event );
         }
         
-        public void reccordProjectFailed( ExecutionEvent event ) {
+        private void recordProjectFailed(ExecutionEvent event) {
             MavenBuildProxy2 mavenBuildProxy2 = getMavenBuildProxy2( event.getProject() );
-            mavenBuildProxy2.end();
             mavenBuildProxy2.setResult( Result.FAILURE );
             MavenProject mavenProject = event.getProject();
             List<MavenReporter> mavenReporters = getMavenReporters( mavenProject );
@@ -491,13 +480,15 @@ public class Maven3Builder extends AbstractMavenBuilder implements DelegatingCal
                     }
                 }
             }
-        }        
+
+            mavenBuildProxy2.end();
+        }
 
         /**
          * @see org.apache.maven.execution.ExecutionListener#mojoSkipped(org.apache.maven.execution.ExecutionEvent)
          */
         public void mojoSkipped( ExecutionEvent event ) {
-            maven3Builder.listener.getLogger().println("mojoSkipped " + event.getMojoExecution().getGroupId() + ":"
+            debug("mojoSkipped " + event.getMojoExecution().getGroupId() + ":"
                                                        + event.getMojoExecution().getArtifactId() + ":"
                                                        + event.getMojoExecution().getVersion()
                                                        + "(" + event.getMojoExecution().getExecutionId() + ")");
@@ -508,24 +499,22 @@ public class Maven3Builder extends AbstractMavenBuilder implements DelegatingCal
          * @see org.apache.maven.execution.ExecutionListener#mojoStarted(org.apache.maven.execution.ExecutionEvent)
          */
         public void mojoStarted( ExecutionEvent event ) {
-            //maven3Builder.listener.getLogger().println("mojoStarted " + event.getMojoExecution().getGroupId() + ":"
-            //                                                          + event.getMojoExecution().getArtifactId() + ":"
-            //                                                          + event.getMojoExecution().getVersion()
-            //                                                          + "(" + event.getMojoExecution().getExecutionId() + ")");
-            reccordMojoStarted( event );
+            debug("mojoStarted " + event.getMojoExecution().getGroupId() + ":"
+                                                                      + event.getMojoExecution().getArtifactId() + ":"
+                                                                      + event.getMojoExecution().getVersion()
+                                                                      + "(" + event.getMojoExecution().getExecutionId() + ")");
+            recordMojoStarted(event);
             this.eventLogger.mojoStarted( event );
         }
         
-        public void reccordMojoStarted( ExecutionEvent event ) {
+        private void recordMojoStarted(ExecutionEvent event) {
             initMojoStartTime( event.getProject() );
             
             MavenProject mavenProject = event.getProject();
             XmlPlexusConfiguration xmlPlexusConfiguration = new XmlPlexusConfiguration( event.getMojoExecution().getConfiguration() );
 
-            Mojo mojo = null;//getMojo( event.getMojoExecution(), event.getSession() );
-            
             MojoInfo mojoInfo =
-                new MojoInfo( event.getMojoExecution(), mojo, xmlPlexusConfiguration,
+                new MojoInfo( event.getMojoExecution(), null, xmlPlexusConfiguration,
                               getExpressionEvaluator( event.getSession(), event.getMojoExecution() ) );
 
             List<MavenReporter> mavenReporters = getMavenReporters( mavenProject );                
@@ -549,15 +538,15 @@ public class Maven3Builder extends AbstractMavenBuilder implements DelegatingCal
          * @see org.apache.maven.execution.ExecutionListener#mojoSucceeded(org.apache.maven.execution.ExecutionEvent)
          */
         public void mojoSucceeded( ExecutionEvent event ) {
-            //maven3Builder.listener.getLogger().println("mojoSucceeded " + event.getMojoExecution().getGroupId() + ":"
-            //                                           + event.getMojoExecution().getArtifactId() + ":"
-            //                                           + event.getMojoExecution().getVersion()
-            //                                           + "(" + event.getMojoExecution().getExecutionId() + ")");
-            reccordMojoSucceeded( event );
+            debug("mojoSucceeded " + event.getMojoExecution().getGroupId() + ":"
+                                                       + event.getMojoExecution().getArtifactId() + ":"
+                                                       + event.getMojoExecution().getVersion()
+                                                       + "(" + event.getMojoExecution().getExecutionId() + ")");
+            recordMojoSucceeded(event);
             this.eventLogger.mojoSucceeded( event );
         }
         
-        public void reccordMojoSucceeded( ExecutionEvent event ) {
+        private void recordMojoSucceeded(ExecutionEvent event) {
             Long startTime = getMojoStartTime( event.getProject() );
             Date endTime = new Date();
             MavenProject mavenProject = event.getProject();
@@ -605,15 +594,21 @@ public class Maven3Builder extends AbstractMavenBuilder implements DelegatingCal
          * @see org.apache.maven.execution.ExecutionListener#mojoFailed(org.apache.maven.execution.ExecutionEvent)
          */
         public void mojoFailed( ExecutionEvent event ) {
-            maven3Builder.listener.getLogger().println("mojoFailed " + event.getMojoExecution().getGroupId() + ":"
+            debug("mojoFailed " + event.getMojoExecution().getGroupId() + ":"
                                                        + event.getMojoExecution().getArtifactId() + ":"
                                                        + event.getMojoExecution().getVersion()
                                                        + "(" + event.getMojoExecution().getExecutionId() + ")");
-            reccordMojoFailed( event );
+            recordMojoFailed(event);
             this.eventLogger.mojoFailed( event );
         }
+
+        private void debug(String msg) {
+            LOGGER.fine(msg);
+            if (DEBUG)
+                maven3Builder.listener.getLogger().println(msg);
+        }
         
-        public void reccordMojoFailed( ExecutionEvent event ) {
+        private void recordMojoFailed(ExecutionEvent event) {
             Long startTime = getMojoStartTime( event.getProject() );
             Date endTime = new Date();
             MavenProject mavenProject = event.getProject();
@@ -667,42 +662,42 @@ public class Maven3Builder extends AbstractMavenBuilder implements DelegatingCal
          */
         public void forkStarted( ExecutionEvent event )
         {
-            maven3Builder.listener.getLogger().println("mojo forkStarted " + event.getMojoExecution().getGroupId() + ":"
+            LOGGER.fine("mojo forkStarted " + event.getMojoExecution().getGroupId() + ":"
                                                        + event.getMojoExecution().getArtifactId() + ":"
                                                        + event.getMojoExecution().getVersion()
                                                        + "(" + event.getMojoExecution().getExecutionId() + ")");
-            reccordMojoStarted( event );
+            recordMojoStarted(event);
         }
 
         /**
          * @see org.apache.maven.execution.ExecutionListener#forkSucceeded(org.apache.maven.execution.ExecutionEvent)
          */
         public void forkSucceeded( ExecutionEvent event ) {
-            maven3Builder.listener.getLogger().println("mojo forkSucceeded " + event.getMojoExecution().getGroupId() + ":"
+            LOGGER.fine("mojo forkSucceeded " + event.getMojoExecution().getGroupId() + ":"
                                                        + event.getMojoExecution().getArtifactId() + ":"
                                                        + event.getMojoExecution().getVersion()
                                                        + "(" + event.getMojoExecution().getExecutionId() + ")");
-            reccordMojoSucceeded( event );
+            recordMojoSucceeded(event);
         }
 
         /**
          * @see org.apache.maven.execution.ExecutionListener#forkFailed(org.apache.maven.execution.ExecutionEvent)
          */
         public void forkFailed( ExecutionEvent event ) {
-            maven3Builder.listener.getLogger().println("mojo forkFailed " + event.getMojoExecution().getGroupId() + ":"
+            LOGGER.fine("mojo forkFailed " + event.getMojoExecution().getGroupId() + ":"
                                                        + event.getMojoExecution().getArtifactId() + ":"
                                                        + event.getMojoExecution().getVersion()
                                                        + "(" + event.getMojoExecution().getExecutionId() + ")");  
-            reccordMojoFailed( event );
+            recordMojoFailed(event);
         }
 
         /**
          * @see org.apache.maven.execution.ExecutionListener#forkedProjectStarted(org.apache.maven.execution.ExecutionEvent)
          */
         public void forkedProjectStarted( ExecutionEvent event ) {
-            maven3Builder.listener.getLogger().println( "forkedProjectStarted " + event.getProject().getGroupId() + ":"
+            LOGGER.fine( "forkedProjectStarted " + event.getProject().getGroupId() + ":"
                                                         + event.getProject().getArtifactId() + event.getProject().getVersion() );
-            reccordProjectStarted( event ); 
+            recordProjectStarted(event);
             this.eventLogger.forkedProjectStarted( event );
         }
 
@@ -710,11 +705,11 @@ public class Maven3Builder extends AbstractMavenBuilder implements DelegatingCal
          * @see org.apache.maven.execution.ExecutionListener#forkedProjectSucceeded(org.apache.maven.execution.ExecutionEvent)
          */
         public void forkedProjectSucceeded( ExecutionEvent event ) {
-            maven3Builder.listener.getLogger().println( "forkedProjectSucceeded "
+            LOGGER.fine( "forkedProjectSucceeded "
                                                         + event.getProject().getGroupId() + ":"
                                                         + event.getProject().getArtifactId()
                                                         + event.getProject().getVersion());
-            reccordProjectSucceeded( event );
+            recordProjectSucceeded(event);
             this.eventLogger.forkedProjectSucceeded( event );
         }
 
@@ -722,16 +717,18 @@ public class Maven3Builder extends AbstractMavenBuilder implements DelegatingCal
          * @see org.apache.maven.execution.ExecutionListener#forkedProjectFailed(org.apache.maven.execution.ExecutionEvent)
          */
         public void forkedProjectFailed( ExecutionEvent event ) {
-            maven3Builder.listener.getLogger().println("forkedProjectFailed " + event.getProject().getGroupId() 
+            LOGGER.fine("forkedProjectFailed " + event.getProject().getGroupId()
                                                        + ":"  + event.getProject().getArtifactId()
                                                        + ":" + event.getProject().getVersion());
-            reccordProjectFailed( event );
-        }        
-        
-    }    
-    
+            recordProjectFailed(event);
+        }
+    }
+
     public static boolean markAsSuccess;
 
-    private static final long serialVersionUID = 1L;    
-    
+    private static final long serialVersionUID = 1L;
+
+    public static boolean DEBUG = false;
+
+    private static final Logger LOGGER = Logger.getLogger(Maven3Builder.class.getName());
 }
